@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand.
 // Implementation
@@ -112,7 +112,7 @@ void OnDemandServerMediaSubsession
     portNumBits serverPortNum;
     if (clientRTCPPort.num() == 0) {
       // We're streaming raw UDP (not RTP). Create a single groupsock:
-      NoReuse dummy; // ensures that we skip over ports that are already in use
+      NoReuse dummy(envir()); // ensures that we skip over ports that are already in use
       for (serverPortNum = fInitialPortNum; ; ++serverPortNum) {
 	struct in_addr dummyAddr; dummyAddr.s_addr = 0;
 
@@ -127,7 +127,7 @@ void OnDemandServerMediaSubsession
     } else {
       // Normal case: We're streaming RTP (over UDP or TCP).  Create a pair of
       // groupsocks (RTP and RTCP), with adjacent port numbers (RTP port number even):
-      NoReuse dummy; // ensures that we skip over ports that are already in use
+      NoReuse dummy(envir()); // ensures that we skip over ports that are already in use
       for (portNumBits serverPortNum = fInitialPortNum; ; serverPortNum += 2) {
 	struct in_addr dummyAddr; dummyAddr.s_addr = 0;
 
@@ -220,8 +220,7 @@ void OnDemandServerMediaSubsession::seekStream(unsigned /*clientSessionId*/,
 					       void* streamToken, double& seekNPT, double streamDuration, u_int64_t& numBytes) {
   numBytes = 0; // by default: unknown
 
-  // Seeking isn't allowed if multiple clients are receiving data from
-  // the same source:
+  // Seeking isn't allowed if multiple clients are receiving data from the same source:
   if (fReuseFirstSource) return;
 
   StreamState* streamState = (StreamState*)streamToken;
@@ -230,6 +229,17 @@ void OnDemandServerMediaSubsession::seekStream(unsigned /*clientSessionId*/,
   }
 }
 
+void OnDemandServerMediaSubsession::seekStream(unsigned /*clientSessionId*/,
+					       void* streamToken, char*& absStart, char*& absEnd) {
+  // Seeking isn't allowed if multiple clients are receiving data from the same source:
+  if (fReuseFirstSource) return;
+
+  StreamState* streamState = (StreamState*)streamToken;
+  if (streamState != NULL && streamState->mediaSource() != NULL) {
+    seekStreamSource(streamState->mediaSource(), absStart, absEnd);
+  }
+}
+
 void OnDemandServerMediaSubsession::setStreamScale(unsigned /*clientSessionId*/,
 						   void* streamToken, float scale) {
   // Changing the scale factor isn't allowed if multiple clients are receiving data
@@ -287,6 +297,11 @@ void OnDemandServerMediaSubsession::seekStreamSource(FramedSource* /*inputSource
   // Default implementation: Do nothing
 }
 
+void OnDemandServerMediaSubsession::seekStreamSource(FramedSource* /*inputSource*/,
+						     char*& /*absStart*/, char*& /*absEnd*/) {
+  // Default implementation: Do nothing
+}
+
 void OnDemandServerMediaSubsession
 ::setStreamSourceScale(FramedSource* /*inputSource*/, float /*scale*/) {
   // Default implementation: Do nothing
