@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // A data structure that represents a session that consists of
 // potentially multiple (audio and/or video) sub-sessions
 // (This data structure is used for media *streamers* - i.e., servers.
@@ -64,9 +64,16 @@ ServerMediaSession::ServerMediaSession(UsageEnvironment& env,
     fSubsessionsTail(NULL), fSubsessionCounter(0),
     fReferenceCount(0), fDeleteWhenUnreferenced(False) {
   fStreamName = strDup(streamName == NULL ? "" : streamName);
-  fInfoSDPString = strDup(info == NULL ? libNameStr : info);
-  fDescriptionSDPString
-    = strDup(description == NULL ? libNameStr : description);
+
+  char* libNamePlusVersionStr = NULL; // by default
+  if (info == NULL || description == NULL) {
+    libNamePlusVersionStr = new char[strlen(libNameStr) + strlen(libVersionStr) + 1];
+    sprintf(libNamePlusVersionStr, "%s%s", libNameStr, libVersionStr);
+  }
+  fInfoSDPString = strDup(info == NULL ? libNamePlusVersionStr : info);
+  fDescriptionSDPString = strDup(description == NULL ? libNamePlusVersionStr : description);
+  delete[] libNamePlusVersionStr;
+
   fMiscSDPLines = strDup(miscSDPLines == NULL ? "" : miscSDPLines);
 
   gettimeofday(&fCreationTime, NULL);
@@ -161,6 +168,12 @@ float ServerMediaSession::duration() const {
   float maxSubsessionDuration = 0.0;
   for (ServerMediaSubsession* subsession = fSubsessionsHead; subsession != NULL;
        subsession = subsession->fNext) {
+    // Hack: If any subsession supports seeking by 'absolute' time, then return a negative value, to indicate that only subsessions
+    // will have a "a=range:" attribute:
+    char* absStartTime = NULL; char* absEndTime = NULL;
+    subsession->getAbsoluteTimeRange(absStartTime, absEndTime);
+    if (absStartTime != NULL) return -1.0f;
+
     float ssduration = subsession->duration();
     if (subsession == fSubsessionsHead) { // this is the first subsession
       minSubsessionDuration = maxSubsessionDuration = ssduration;
@@ -212,10 +225,10 @@ char* ServerMediaSession::generateSDPDescription() {
     for (subsession = fSubsessionsHead; subsession != NULL;
 	 subsession = subsession->fNext) {
       char const* sdpLines = subsession->sdpLines();
-      if (sdpLines == NULL) break; // the media's not available
+      if (sdpLines == NULL) continue; // the media's not available
       sdpLength += strlen(sdpLines);
     }
-    if (subsession != NULL) break; // an error occurred
+    if (sdpLength == 0) break; // the session has no usable subsessions
 
     // Unless subsessions have differing durations, we also have a "a=range:" line:
     float dur = duration();
@@ -275,7 +288,8 @@ char* ServerMediaSession::generateSDPDescription() {
     for (subsession = fSubsessionsHead; subsession != NULL;
 	 subsession = subsession->fNext) {
       mediaSDP += strlen(mediaSDP);
-      sprintf(mediaSDP, "%s", subsession->sdpLines());
+      char const* sdpLines = subsession->sdpLines();
+      if (sdpLines != NULL) sprintf(mediaSDP, "%s", sdpLines);
     }
   } while (0);
 
@@ -341,6 +355,10 @@ void ServerMediaSubsession::seekStream(unsigned /*clientSessionId*/,
   // default implementation: do nothing
   numBytes = 0;
 }
+void ServerMediaSubsession::seekStream(unsigned /*clientSessionId*/,
+				       void* /*streamToken*/, char*& /*absStart*/, char*& /*absEnd*/) {
+  // default implementation: do nothing
+}
 FramedSource* ServerMediaSubsession::getStreamSource(void* /*streamToken*/) {
   // default implementation: return NULL
   return NULL;
@@ -364,6 +382,11 @@ float ServerMediaSubsession::duration() const {
   return 0.0;
 }
 
+void ServerMediaSubsession::getAbsoluteTimeRange(char*& absStartTime, char*& absEndTime) const {
+  // default implementation: We don't support seeking by 'absolute' time, so indicate this by setting both parameters to NULL:
+  absStartTime = absEndTime = NULL;
+}
+
 void ServerMediaSubsession::setServerAddressAndPortForSDP(netAddressBits addressBits,
 							  portNumBits portBits) {
   fServerAddressForSDP = addressBits;
@@ -372,6 +395,20 @@ void ServerMediaSubsession::setServerAddressAndPortForSDP(netAddressBits address
 
 char const*
 ServerMediaSubsession::rangeSDPLine() const {
+  // First, check for the special case where we support seeking by 'absolute' time:
+  char* absStart = NULL; char* absEnd = NULL;
+  getAbsoluteTimeRange(absStart, absEnd);
+  if (absStart != NULL) {
+    char buf[100];
+
+    if (absEnd != NULL) {
+      sprintf(buf, "a=range:clock=%s-%s\r\n", absStart, absEnd);
+    } else {
+      sprintf(buf, "a=range:clock=%s-\r\n", absStart);
+    }
+    return strDup(buf);
+  }
+
   if (fParentSession == NULL) return NULL;
 
   // If all of our parent's subsessions have the same duration
